@@ -4,16 +4,16 @@ namespace MediaWiki\Extension\AbuseFilter\Special;
 
 use DifferenceEngine;
 use InvalidArgumentException;
-use ManualLogEntry;
 use MediaWiki\Cache\LinkBatchFactory;
+use MediaWiki\Extension\AbuseFilter\AbuseFilter;
 use MediaWiki\Extension\AbuseFilter\AbuseFilterPermissionManager;
 use MediaWiki\Extension\AbuseFilter\AbuseFilterServices;
 use MediaWiki\Extension\AbuseFilter\AbuseLoggerFactory;
 use MediaWiki\Extension\AbuseFilter\CentralDBNotAvailableException;
 use MediaWiki\Extension\AbuseFilter\Consequences\ConsequencesRegistry;
 use MediaWiki\Extension\AbuseFilter\Filter\FilterNotFoundException;
-use MediaWiki\Extension\AbuseFilter\Filter\Flags;
-use MediaWiki\Extension\AbuseFilter\FilterUtils;
+use MediaWiki\Extension\AbuseFilter\Filter\MutableFilter;
+use MediaWiki\Extension\AbuseFilter\FilterLookup;
 use MediaWiki\Extension\AbuseFilter\GlobalNameUtils;
 use MediaWiki\Extension\AbuseFilter\Pager\AbuseLogPager;
 use MediaWiki\Extension\AbuseFilter\SpecsFormatter;
@@ -27,6 +27,7 @@ use MediaWiki\Html\Html;
 use MediaWiki\Html\ListToggle;
 use MediaWiki\HTMLForm\HTMLForm;
 use MediaWiki\Linker\Linker;
+use MediaWiki\Logging\ManualLogEntry;
 use MediaWiki\MediaWikiServices;
 use MediaWiki\Permissions\Authority;
 use MediaWiki\Permissions\PermissionManager;
@@ -40,9 +41,7 @@ use MediaWiki\User\UserIdentityLookup;
 use MediaWiki\WikiMap\WikiMap;
 use OOUI\ButtonInputWidget;
 use stdClass;
-use Wikimedia\Rdbms\IExpression;
 use Wikimedia\Rdbms\LBFactory;
-use Wikimedia\Rdbms\LikeValue;
 
 class SpecialAbuseLog extends AbuseFilterSpecialPage {
 	public const PAGE_NAME = 'AbuseLog';
@@ -107,48 +106,18 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 	/** @var string|null The filter group to search, as defined in $wgAbuseFilterValidGroups */
 	private $mSearchGroup;
 
-	/** @var LBFactory */
-	private $lbFactory;
-
-	/** @var LinkBatchFactory */
-	private $linkBatchFactory;
-
-	/** @var PermissionManager */
-	private $permissionManager;
-
-	/** @var UserIdentityLookup */
-	private $userIdentityLookup;
-
-	/** @var ConsequencesRegistry */
-	private $consequencesRegistry;
-
-	/** @var VariablesBlobStore */
-	private $varBlobStore;
-
-	/** @var SpecsFormatter */
-	private $specsFormatter;
-
-	/** @var VariablesFormatter */
-	private $variablesFormatter;
-
-	/** @var VariablesManager */
-	private $varManager;
-
+	private LBFactory $lbFactory;
+	private LinkBatchFactory $linkBatchFactory;
+	private PermissionManager $permissionManager;
+	private UserIdentityLookup $userIdentityLookup;
+	private ConsequencesRegistry $consequencesRegistry;
+	private VariablesBlobStore $varBlobStore;
+	private SpecsFormatter $specsFormatter;
+	private VariablesFormatter $variablesFormatter;
+	private VariablesManager $varManager;
 	private AbuseLoggerFactory $abuseLoggerFactory;
+	private FilterLookup $filterLookup;
 
-	/**
-	 * @param LBFactory $lbFactory
-	 * @param LinkBatchFactory $linkBatchFactory
-	 * @param PermissionManager $permissionManager
-	 * @param UserIdentityLookup $userIdentityLookup
-	 * @param AbuseFilterPermissionManager $afPermissionManager
-	 * @param ConsequencesRegistry $consequencesRegistry
-	 * @param VariablesBlobStore $varBlobStore
-	 * @param SpecsFormatter $specsFormatter
-	 * @param VariablesFormatter $variablesFormatter
-	 * @param VariablesManager $varManager
-	 * @param AbuseLoggerFactory $abuseLoggerFactory
-	 */
 	public function __construct(
 		LBFactory $lbFactory,
 		LinkBatchFactory $linkBatchFactory,
@@ -160,7 +129,8 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		SpecsFormatter $specsFormatter,
 		VariablesFormatter $variablesFormatter,
 		VariablesManager $varManager,
-		AbuseLoggerFactory $abuseLoggerFactory
+		AbuseLoggerFactory $abuseLoggerFactory,
+		FilterLookup $filterLookup
 	) {
 		parent::__construct( self::PAGE_NAME, 'abusefilter-log', $afPermissionManager );
 		$this->lbFactory = $lbFactory;
@@ -175,6 +145,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		$this->variablesFormatter->setMessageLocalizer( $this );
 		$this->varManager = $varManager;
 		$this->abuseLoggerFactory = $abuseLoggerFactory;
+		$this->filterLookup = $filterLookup;
 	}
 
 	/**
@@ -422,6 +393,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			->setWrapperLegendMsg( 'abusefilter-log-search' )
 			->setSubmitTextMsg( 'abusefilter-log-search-submit' )
 			->setMethod( 'get' )
+			->setId( 'abusefilter-log-search' )
 			->setCollapsibleOptions( true )
 			->prepareForm()
 			->displayForm( false );
@@ -433,6 +405,9 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			$this->afPermissionManager,
 			$this->getContext(),
 			$this->getLinkRenderer(),
+			$this->linkBatchFactory,
+			$this->permissionManager,
+			$this->varBlobStore,
 			self::PAGE_NAME
 		);
 		$view->show();
@@ -504,51 +479,42 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 				}
 			}
 
-			// if a filter is hidden, users who can't view private filters should
-			// not be able to find log entries generated by it.
-			if ( !$this->afPermissionManager->canViewPrivateFiltersLogs( $performer ) ) {
-				$searchedForPrivate = false;
-				foreach ( $filtersList as $index => $filterData ) {
-					try {
-						$filter = AbuseFilterServices::getFilterLookup()->getFilter( ...$filterData );
-					} catch ( FilterNotFoundException $_ ) {
-						unset( $filtersList[$index] );
-						$foundInvalid = true;
-						continue;
-					}
-					if ( $filter->isHidden() ) {
-						unset( $filtersList[$index] );
-						$searchedForPrivate = true;
-					}
+			// If the user cannot see the filter because it's protected or hidden, then
+			// don't show logs associated with it.
+			$canViewPrivateFilters = $this->afPermissionManager->canViewPrivateFiltersLogs( $performer );
+			$searchedForProtectedWhenLacksPermission = false;
+			$searchedForHiddenWhenLacksPermission = false;
+			foreach ( $filtersList as $index => $filterData ) {
+				try {
+					$filter = $this->filterLookup->getFilter( ...$filterData );
+				} catch ( FilterNotFoundException $_ ) {
+					unset( $filtersList[$index] );
+					$foundInvalid = true;
+					continue;
 				}
-				if ( $searchedForPrivate ) {
-					$out->addWikiMsg( 'abusefilter-log-private-not-included' );
+
+				if ( $filter->isHidden() && !$canViewPrivateFilters ) {
+					unset( $filtersList[$index] );
+					$searchedForHiddenWhenLacksPermission = true;
+				}
+				if (
+					$filter->isProtected() &&
+					!$this->afPermissionManager->canViewProtectedVariablesInFilter( $performer, $filter )->isGood()
+				) {
+					unset( $filtersList[$index] );
+					$searchedForProtectedWhenLacksPermission = true;
 				}
 			}
 
-			// if a filter is protected, users who can't view protected filters should
-			// not be able to find log entries generated by it.
-			if ( !$this->afPermissionManager->canViewProtectedVariables( $performer ) ) {
-				$searchedForProtected = false;
-				foreach ( $filtersList as $index => $filterData ) {
-					try {
-						$filter = AbuseFilterServices::getFilterLookup()->getFilter( ...$filterData );
-					} catch ( FilterNotFoundException $_ ) {
-						unset( $filtersList[$index] );
-						$foundInvalid = true;
-						continue;
-					}
-					if ( $filter->isProtected() ) {
-						unset( $filtersList[$index] );
-						$searchedForProtected = true;
-					}
-				}
-				if ( $searchedForProtected ) {
-					$out->addWikiMsg( 'abusefilter-log-protected-not-included' );
-				}
+			if ( $searchedForProtectedWhenLacksPermission ) {
+				$out->addWikiMsg( 'abusefilter-log-protected-not-included' );
+			}
+			if ( $searchedForHiddenWhenLacksPermission ) {
+				$out->addWikiMsg( 'abusefilter-log-private-not-included' );
 			}
 
 			if ( $foundInvalid ) {
+				$out->addModuleStyles( 'mediawiki.codex.messagebox.styles' );
 				// @todo Tell what the invalid IDs are
 				$out->addHTML(
 					Html::rawElement(
@@ -625,18 +591,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 
 		if ( $this->mSearchActionTaken ) {
 			if ( in_array( $this->mSearchActionTaken, $this->consequencesRegistry->getAllActionNames() ) ) {
-				$conds[] = $dbr->expr( 'afl_actions', '=', $this->mSearchActionTaken )
-					->or( 'afl_actions', IExpression::LIKE, new LikeValue(
-						$this->mSearchActionTaken, ',', $dbr->anyString()
-					) )
-					->or( 'afl_actions', IExpression::LIKE, new LikeValue(
-						$dbr->anyString(), ',', $this->mSearchActionTaken
-					) )
-					->or( 'afl_actions', IExpression::LIKE, new LikeValue(
-						$dbr->anyString(),
-						',', $this->mSearchActionTaken, ',',
-						$dbr->anyString()
-					) );
+				$conds[] = AbuseFilter::findInSet( $dbr, 'afl_actions', $this->mSearchActionTaken );
 			} elseif ( $this->mSearchActionTaken === 'noactions' ) {
 				$conds['afl_actions'] = '';
 			}
@@ -658,6 +613,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			$this->linkBatchFactory,
 			$this->permissionManager,
 			$this->afPermissionManager,
+			$this->varBlobStore,
 			$this->getName()
 		);
 		$pager->doQuery();
@@ -725,6 +681,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			$this->linkBatchFactory,
 			$this->permissionManager,
 			$this->afPermissionManager,
+			$this->varBlobStore,
 			$this->getName()
 		);
 
@@ -743,44 +700,32 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			->joinConds( $join_conds )
 			->fetchRow();
 
-		$error = null;
-		$privacyLevel = Flags::FILTER_PUBLIC;
 		if ( !$row ) {
-			$error = 'abusefilter-log-nonexistent';
+			$out->addWikiMsg( 'abusefilter-log-nonexistent' );
+			return;
+		}
+
+		$error = null;
+		$filterID = $row->afl_filter_id;
+		$global = $row->afl_global;
+
+		try {
+			$filter = $this->filterLookup->getFilter( $filterID, $global );
+		} catch ( CentralDBNotAvailableException $_ ) {
+			// Conservatively assume that it's hidden and protected, like in AbuseLogPager::doFormatRow
+			$filter = MutableFilter::newDefault();
+			$filter->setHidden( true );
+			$filter->setProtected( true );
+		}
+
+		if ( !$this->afPermissionManager->canSeeLogDetailsForFilter( $performer, $filter ) ) {
+			$error = 'abusefilter-log-cannot-see-details';
 		} else {
-			$filterID = $row->afl_filter_id;
-			$global = $row->afl_global;
-
-			$privacyLevel = $row->af_hidden;
-			if ( $global ) {
-				try {
-					$privacyLevel = AbuseFilterServices::getFilterLookup()->getFilter( $filterID, $global )
-						->getPrivacyLevel();
-				} catch ( CentralDBNotAvailableException $_ ) {
-					// Conservatively assume that it's hidden and protected, like in AbuseLogPager::doFormatRow
-					$privacyLevel = Flags::FILTER_HIDDEN & Flags::FILTER_USES_PROTECTED_VARS;
-				}
-			}
-
-			if ( !$this->afPermissionManager->canSeeLogDetailsForFilter( $performer, $privacyLevel ) ) {
-				$error = 'abusefilter-log-cannot-see-details';
-			} else {
-				$visibility = self::getEntryVisibilityForUser( $row, $performer, $this->afPermissionManager );
-				if ( $visibility === self::VISIBILITY_HIDDEN ) {
-					$error = 'abusefilter-log-details-hidden';
-				} elseif ( $visibility === self::VISIBILITY_HIDDEN_IMPLICIT ) {
-					$error = 'abusefilter-log-details-hidden-implicit';
-				}
-			}
-
-			// Only show the preference error if another error isn't already set
-			// as this error shouldn't take precedence over a view permission error
-			if (
-				FilterUtils::isProtected( $privacyLevel ) &&
-				!$this->afPermissionManager->canViewProtectedVariableValues( $performer ) &&
-				!$error
-			) {
-				$error = 'abusefilter-examine-protected-vars-permission';
+			$visibility = self::getEntryVisibilityForUser( $row, $performer, $this->afPermissionManager );
+			if ( $visibility === self::VISIBILITY_HIDDEN ) {
+				$error = 'abusefilter-log-details-hidden';
+			} elseif ( $visibility === self::VISIBILITY_HIDDEN_IMPLICIT ) {
+				$error = 'abusefilter-log-details-hidden-implicit';
 			}
 		}
 
@@ -789,6 +734,20 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 			return;
 		}
 
+		// Load data
+		$vars = $this->varBlobStore->loadVarDump( $row );
+		$varsArray = $this->varManager->dumpAllVars( $vars, true );
+
+		// Prevent users seeing logs which contain protected variables that the user cannot see.
+		if (
+			$filter->isProtected() &&
+			!$this->afPermissionManager->canViewProtectedVariables( $performer, array_keys( $varsArray ) )
+				->isGood()
+		) {
+			$out->addWikiMsg( 'abusefilter-examine-protected-vars-permission' );
+			return;
+		}
+
 		$output = Html::element(
 			'legend',
 			[],
@@ -798,48 +757,36 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		);
 		$output .= Html::rawElement( 'p', [], $pager->doFormatRow( $row, false ) );
 
-		// Load data
-		$vars = $this->varBlobStore->loadVarDump( $row );
-		$varsArray = $this->varManager->dumpAllVars( $vars, true );
-		$shouldLogProtectedVarAccess = false;
-
-		// If a non-protected filter and a protected filter have overlapping conditions,
-		// it's possible for a hit to contain a protected variable and for that variable
-		// to be dumped and displayed on a detail page that wouldn't be considered
-		// protected (because it caught on the public filter).
+		// AbuseFilter logs created before T390086 may have protected variables present in the variable dump
+		// when the filter itself isn't protected. This is because a different filter matched against the
+		// a protected variable which caused the value to be added to the var dump for the public filter
+		// match.
 		// We shouldn't block access to the details of an otherwise public filter hit so
 		// instead only check for access to the protected variables and redact them if the user
 		// shouldn't see them.
 		$userAuthority = $this->getAuthority();
-		$canViewProtectedVars = $this->afPermissionManager->canViewProtectedVariableValues( $userAuthority );
+		$protectedVariableValuesShown = [];
 		foreach ( $this->afPermissionManager->getProtectedVariables() as $protectedVariable ) {
 			if ( isset( $varsArray[$protectedVariable] ) ) {
-				if ( !$canViewProtectedVars ) {
+				// Try each variable at a time, as the user may be able to see some but not all of the
+				// protected variables. We only want to redact what is necessary to redact.
+				$canViewProtectedVariable = $this->afPermissionManager
+					->canViewProtectedVariables( $userAuthority, [ $protectedVariable ] )->isGood();
+				if ( !$canViewProtectedVariable ) {
 					$varsArray[$protectedVariable] = '';
 				} else {
-					// Protected variables in protected filters logs access in the general permission check
-					// Log access to non-protected filters that happen to expose protected variables here
-					if ( !FilterUtils::isProtected( $privacyLevel ) ) {
-						$shouldLogProtectedVarAccess = true;
-					}
+					$protectedVariableValuesShown[] = $protectedVariable;
 				}
 			}
 		}
 		$vars = VariableHolder::newFromArray( $varsArray );
 
-		// Log if protected variables are accessed
-		if (
-			FilterUtils::isProtected( $privacyLevel ) &&
-			$canViewProtectedVars
-		) {
-			$shouldLogProtectedVarAccess = true;
-		}
-
-		if ( $shouldLogProtectedVarAccess ) {
+		if ( $filter->isProtected() ) {
 			$logger = $this->abuseLoggerFactory->getProtectedVarsAccessLogger();
 			$logger->logViewProtectedVariableValue(
 				$userAuthority->getUser(),
-				$varsArray['user_name'] ?? $varsArray['accountname']
+				$varsArray['user_name'] ?? $varsArray['accountname'],
+				$protectedVariableValuesShown
 			);
 		}
 
@@ -921,36 +868,28 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		$dbr = MediaWikiServices::getInstance()->getConnectionProvider()->getReplicaDatabase();
 
 		$row = $dbr->newSelectQueryBuilder()
-			->select( [ 'afl_id', 'afl_user_text', 'afl_filter_id', 'afl_global', 'afl_timestamp', 'afl_ip',
-				'af_id', 'af_public_comments', 'af_hidden' ] )
+			->select( [ 'afl_id', 'afl_user_text', 'afl_filter_id', 'afl_global', 'afl_timestamp', 'afl_ip' ] )
 			->from( 'abuse_filter_log' )
-			->leftJoin( 'abuse_filter', null, [ 'af_id=afl_filter_id', 'afl_global' => 0 ] )
 			->where( [ 'afl_id' => $id ] )
 			->caller( __METHOD__ )
 			->fetchRow();
 
-		$status = Status::newGood();
 		if ( !$row ) {
-			$status->fatal( 'abusefilter-log-nonexistent' );
-			return $status;
+			return Status::newFatal( 'abusefilter-log-nonexistent' );
 		}
 
-		$filterID = $row->afl_filter_id;
-		$global = $row->afl_global;
-
-		if ( $global ) {
-			$lookup = AbuseFilterServices::getFilterLookup();
-			$privacyLevel = $lookup->getFilter( $filterID, $global )->getPrivacyLevel();
-		} else {
-			$privacyLevel = $row->af_hidden;
+		$filter = AbuseFilterServices::getFilterLookup()->getFilter( $row->afl_filter_id, $row->afl_global );
+		if ( !$afPermissionManager->canSeeLogDetailsForFilter( $authority, $filter ) ) {
+			return Status::newFatal( 'abusefilter-log-cannot-see-details' );
 		}
 
-		if ( !$afPermissionManager->canSeeLogDetailsForFilter( $authority, $privacyLevel ) ) {
-			$status->fatal( 'abusefilter-log-cannot-see-details' );
-			return $status;
-		}
-		$status->setResult( true, $row );
-		return $status;
+		// Because FilterLookup::getFilter gets the data for us, don't duplicate queries and use the data
+		// from the ExistingFilter object to populate information about the filter in the $row.
+		$row->af_id = (string)$filter->getId();
+		$row->af_public_comments = $filter->getName();
+		$row->af_hidden = $filter->getPrivacyLevel();
+
+		return Status::newGood( $row );
 	}
 
 	/**
@@ -1112,6 +1051,7 @@ class SpecialAbuseLog extends AbuseFilterSpecialPage {
 		// Make sure it is a valid request
 		$token = $request->getVal( 'wpEditToken' );
 		if ( !$request->wasPosted() || !$user->matchEditToken( $token ) ) {
+			$out->addModuleStyles( 'mediawiki.codex.messagebox.styles' );
 			$out->addHTML(
 				Html::rawElement(
 					'p',
