@@ -1,31 +1,66 @@
 <?php
 
+use MediaWiki\Content\ContentHandler;
+use MediaWiki\Content\CssContentHandler;
+use MediaWiki\Content\JavaScriptContentHandler;
+use MediaWiki\Content\JsonContent;
+use MediaWiki\Content\JsonContentHandler;
+use MediaWiki\Content\TextContentHandler;
 use MediaWiki\Content\ValidationParams;
+use MediaWiki\Content\WikitextContent;
+use MediaWiki\Content\WikitextContentHandler;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\Exception\MWException;
+use MediaWiki\Language\Language;
+use MediaWiki\Languages\LanguageNameUtils;
+use MediaWiki\Linker\LinkRenderer;
+use MediaWiki\MainConfigNames;
+use MediaWiki\Page\Hook\OpportunisticLinksUpdateHook;
 use MediaWiki\Page\PageIdentity;
 use MediaWiki\Page\PageIdentityValue;
+use MediaWiki\Page\WikiPage;
+use MediaWiki\Parser\MagicWordFactory;
+use MediaWiki\Parser\ParserFactory;
+use MediaWiki\Parser\ParserOutput;
+use MediaWiki\Parser\Parsoid\ParsoidParserFactory;
+use MediaWiki\Title\Title;
+use MediaWiki\Title\TitleFactory;
 use Wikimedia\TestingAccessWrapper;
+use Wikimedia\UUID\GlobalIdGenerator;
 
 /**
  * @group ContentHandler
  * @group Database
+ * @covers \MediaWiki\Content\ContentHandler
  */
 class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 
 	protected function setUp(): void {
 		parent::setUp();
 
-		$this->setMwGlobals( [
-			'wgExtraNamespaces' => [
+		$this->overrideConfigValues( [
+			MainConfigNames::ExtraNamespaces => [
 				12312 => 'Dummy',
 				12313 => 'Dummy_talk',
 			],
 			// The below tests assume that namespaces not mentioned here (Help, User, MediaWiki, ..)
 			// default to CONTENT_MODEL_WIKITEXT.
-			'wgNamespaceContentModels' => [
+			MainConfigNames::NamespaceContentModels => [
 				12312 => 'testing',
 			],
-			'wgContentHandlers' => [
-				CONTENT_MODEL_WIKITEXT => WikitextContentHandler::class,
+			MainConfigNames::ContentHandlers => [
+				CONTENT_MODEL_WIKITEXT => [
+					'class' => WikitextContentHandler::class,
+					'services' => [
+						'TitleFactory',
+						'ParserFactory',
+						'GlobalIdGenerator',
+						'LanguageNameUtils',
+						'LinkRenderer',
+						'MagicWordFactory',
+						'ParsoidParserFactory',
+					],
+				],
 				CONTENT_MODEL_JAVASCRIPT => JavaScriptContentHandler::class,
 				CONTENT_MODEL_JSON => JsonContentHandler::class,
 				CONTENT_MODEL_CSS => CssContentHandler::class,
@@ -36,16 +71,6 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 				}
 			],
 		] );
-
-		// Reset LinkCache
-		$this->getServiceContainer()->resetServiceForTesting( 'LinkCache' );
-	}
-
-	protected function tearDown(): void {
-		// Reset LinkCache
-		$this->getServiceContainer()->resetServiceForTesting( 'LinkCache' );
-
-		parent::tearDown();
 	}
 
 	public function addDBDataOnce() {
@@ -53,7 +78,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$this->insertPage( 'Smithee', 'A smithee is one who smiths. See also [[Alan Smithee]]' );
 	}
 
-	public static function dataGetDefaultModelFor() {
+	public static function provideGetDefaultModelFor() {
 		return [
 			[ 'Help:Foo', CONTENT_MODEL_WIKITEXT ],
 			[ 'Help:Foo.js', CONTENT_MODEL_WIKITEXT ],
@@ -82,27 +107,15 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @dataProvider dataGetDefaultModelFor
-	 * @covers ContentHandler::getDefaultModelFor
+	 * @dataProvider provideGetDefaultModelFor
 	 */
 	public function testGetDefaultModelFor( $title, $expectedModelId ) {
 		$title = Title::newFromText( $title );
+		$this->hideDeprecated( 'MediaWiki\\Content\\ContentHandler::getDefaultModelFor' );
 		$this->assertEquals( $expectedModelId, ContentHandler::getDefaultModelFor( $title ) );
 	}
 
-	/**
-	 * @dataProvider dataGetDefaultModelFor
-	 * @covers ContentHandler::getForTitle
-	 */
-	public function testGetForTitle( $title, $expectedContentModel ) {
-		$this->hideDeprecated( 'ContentHandler::getForTitle' );
-		$title = Title::newFromText( $title );
-		$this->getServiceContainer()->getLinkCache()->addBadLinkObj( $title );
-		$handler = ContentHandler::getForTitle( $title );
-		$this->assertEquals( $expectedContentModel, $handler->getModelID() );
-	}
-
-	public static function dataGetLocalizedName() {
+	public static function provideGetLocalizedName() {
 		return [
 			[ null, null ],
 			[ "xyzzy", null ],
@@ -113,8 +126,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @dataProvider dataGetLocalizedName
-	 * @covers ContentHandler::getLocalizedName
+	 * @dataProvider provideGetLocalizedName
 	 */
 	public function testGetLocalizedName( $id, $expected ) {
 		$name = ContentHandler::getLocalizedName( $id );
@@ -131,7 +143,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		}
 	}
 
-	public static function dataGetPageLanguage() {
+	public static function provideGetPageLanguage() {
 		global $wgLanguageCode;
 
 		return [
@@ -146,8 +158,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @dataProvider dataGetPageLanguage
-	 * @covers ContentHandler::getPageLanguage
+	 * @dataProvider provideGetPageLanguage
 	 */
 	public function testGetPageLanguage( $title, $expected ) {
 		$title = Title::newFromText( $title );
@@ -162,87 +173,28 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$this->assertEquals( $expected, $lang->getCode() );
 	}
 
-	public static function dataGetContentText_Null() {
-		return [
-			[ 'fail' ],
-			[ 'serialize' ],
-			[ 'ignore' ],
-		];
-	}
-
-	/**
-	 * @dataProvider dataGetContentText_Null
-	 * @covers ContentHandler::getContentText
-	 */
-	public function testGetContentText_Null( $contentHandlerTextFallback ) {
-		$this->setMwGlobals( 'wgContentHandlerTextFallback', $contentHandlerTextFallback );
-
+	public function testGetContentText_Null() {
+		$this->hideDeprecated( 'MediaWiki\\Content\\ContentHandler::getContentText' );
 		$content = null;
-
 		$text = ContentHandler::getContentText( $content );
 		$this->assertSame( '', $text );
 	}
 
-	public static function dataGetContentText_TextContent() {
-		return [
-			[ 'fail' ],
-			[ 'serialize' ],
-			[ 'ignore' ],
-		];
-	}
-
-	/**
-	 * @dataProvider dataGetContentText_TextContent
-	 * @covers ContentHandler::getContentText
-	 */
-	public function testGetContentText_TextContent( $contentHandlerTextFallback ) {
-		$this->setMwGlobals( 'wgContentHandlerTextFallback', $contentHandlerTextFallback );
-
+	public function testGetContentText_TextContent() {
+		$this->hideDeprecated( 'MediaWiki\\Content\\ContentHandler::getContentText' );
 		$content = new WikitextContent( "hello world" );
-
 		$text = ContentHandler::getContentText( $content );
 		$this->assertEquals( $content->getText(), $text );
 	}
 
-	/**
-	 * ContentHandler::getContentText should have thrown an exception for non-text Content object
-	 *
-	 * @covers ContentHandler::getContentText
-	 */
-	public function testGetContentText_NonTextContent_fail() {
-		$this->setMwGlobals( 'wgContentHandlerTextFallback', 'fail' );
-
+	public function testGetContentText_NonTextContent() {
+		$this->hideDeprecated( 'MediaWiki\\Content\\ContentHandler::getContentText' );
 		$content = new DummyContentForTesting( "hello world" );
-
-		$this->expectException( MWException::class );
-		ContentHandler::getContentText( $content );
-	}
-
-	/**
-	 * @covers ContentHandler::getContentText
-	 */
-	public function testGetContentText_NonTextContent_serialize() {
-		$this->setMwGlobals( 'wgContentHandlerTextFallback', 'serialize' );
-
-		$content = new DummyContentForTesting( "hello world" );
-
-		$text = ContentHandler::getContentText( $content );
-		$this->assertEquals( $content->serialize(), $text );
-	}
-
-	/**
-	 * @covers ContentHandler::getContentText
-	 */
-	public function testGetContentText_NonTextContent_ignore() {
-		$this->setMwGlobals( 'wgContentHandlerTextFallback', 'ignore' );
-
-		$content = new DummyContentForTesting( "hello world" );
-
 		$text = ContentHandler::getContentText( $content );
 		$this->assertNull( $text );
 	}
 
-	public static function dataMakeContent() {
+	public static function provideMakeContent() {
 		return [
 			[ 'hallo', 'Help:Test', null, null, CONTENT_MODEL_WIKITEXT, false ],
 			[ 'hallo', 'MediaWiki:Test.js', null, null, CONTENT_MODEL_JAVASCRIPT, false ],
@@ -291,8 +243,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @dataProvider dataMakeContent
-	 * @covers ContentHandler::makeContent
+	 * @dataProvider provideMakeContent
 	 */
 	public function testMakeContent( $data, $title, $modelId, $format,
 		$expectedModelId, $shouldFail
@@ -319,16 +270,13 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers ContentHandler::getAutosummary
-	 *
-	 * Test if we become a "Created blank page" summary from getAutoSummary if no Content added to
-	 * page.
+	 * getAutoSummary() should set "Created blank page" summary if we save an empy string.
 	 */
 	public function testGetAutosummary() {
 		$this->setContentLang( 'en' );
 
 		$content = new DummyContentHandlerForTesting( CONTENT_MODEL_WIKITEXT );
-		$title = Title::newFromText( 'Help:Test' );
+		$title = Title::makeTitle( NS_HELP, 'Test' );
 		// Create a new content object with no content
 		$newContent = ContentHandler::makeContent( '', $title, CONTENT_MODEL_WIKITEXT, null );
 		// first check, if we become a blank page created summary with the right bitmask
@@ -344,10 +292,9 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 
 	/**
 	 * Test software tag that is added when content model of the page changes
-	 * @covers ContentHandler::getChangeTag
 	 */
 	public function testGetChangeTag() {
-		$this->setMwGlobals( 'wgSoftwareTags', [ 'mw-contentmodelchange' => true ] );
+		$this->overrideConfigValue( MainConfigNames::SoftwareTags, [ 'mw-contentmodelchange' => true ] );
 		$wikitextContentHandler = new DummyContentHandlerForTesting( CONTENT_MODEL_WIKITEXT );
 		// Create old content object with javascript content model
 		$oldContent = ContentHandler::makeContent( '', null, CONTENT_MODEL_JAVASCRIPT, null );
@@ -358,17 +305,11 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$this->assertSame( 'mw-contentmodelchange', $tag );
 	}
 
-	/**
-	 * @covers ContentHandler::supportsCategories
-	 */
 	public function testSupportsCategories() {
 		$handler = new DummyContentHandlerForTesting( CONTENT_MODEL_WIKITEXT );
 		$this->assertTrue( $handler->supportsCategories(), 'content model supports categories' );
 	}
 
-	/**
-	 * @covers ContentHandler::supportsDirectEditing
-	 */
 	public function testSupportsDirectEditing() {
 		$handler = new DummyContentHandlerForTesting( CONTENT_MODEL_JSON );
 		$this->assertFalse( $handler->supportsDirectEditing(), 'direct editing is not supported' );
@@ -384,7 +325,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		return true;
 	}
 
-	public function provideGetModelForID() {
+	public static function provideGetModelForID() {
 		return [
 			[ CONTENT_MODEL_WIKITEXT, WikitextContentHandler::class ],
 			[ CONTENT_MODEL_JAVASCRIPT, JavaScriptContentHandler::class ],
@@ -397,22 +338,24 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers ContentHandler::getForModelID
 	 * @dataProvider provideGetModelForID
 	 */
 	public function testGetModelForID( $modelId, $handlerClass ) {
-		$handler = ContentHandler::getForModelID( $modelId );
+		$handler = $this->getServiceContainer()->getContentHandlerFactory()
+			->getContentHandler( $modelId );
 
 		$this->assertInstanceOf( $handlerClass, $handler );
 	}
 
-	/**
-	 * @covers ContentHandler::getFieldsForSearchIndex
-	 */
 	public function testGetFieldsForSearchIndex() {
 		$searchEngine = $this->newSearchEngine();
 
-		$handler = ContentHandler::getForModelID( CONTENT_MODEL_WIKITEXT );
+		$handler = $this->getMockBuilder( ContentHandler::class )
+			->onlyMethods(
+				[ 'serializeContent', 'unserializeContent', 'makeEmptyContent' ]
+			)
+			->disableOriginalConstructor()
+			->getMock();
 
 		$fields = $handler->getFieldsForSearchIndex( $searchEngine );
 
@@ -424,24 +367,20 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	private function newSearchEngine() {
-		$searchEngine = $this->getMockBuilder( SearchEngine::class )
-			->getMock();
+		$searchEngine = $this->createMock( SearchEngine::class );
 
 		$searchEngine->method( 'makeSearchFieldMapping' )
-			->will( $this->returnCallback( static function ( $name, $type ) {
+			->willReturnCallback( static function ( $name, $type ) {
 					return new DummySearchIndexFieldDefinition( $name, $type );
-			} ) );
+			} );
 
 		return $searchEngine;
 	}
 
-	/**
-	 * @covers ContentHandler::getDataForSearchIndex
-	 */
 	public function testDataIndexFields() {
 		$mockEngine = $this->createMock( SearchEngine::class );
-		$title = Title::newFromText( 'Not_Main_Page', NS_MAIN );
-		$page = new WikiPage( $title );
+		$title = Title::makeTitle( NS_MAIN, 'Not_Main_Page' );
+		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $title );
 
 		$this->setTemporaryHook( 'SearchDataForIndex',
 			static function (
@@ -454,9 +393,9 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 				$fields['testDataField'] = 'test content';
 			} );
 
-		$contentRenderer = $this->getServiceContainer()->getContentRenderer();
-		$output = $contentRenderer->getParserOutput( $page->getContent(), $title );
-		$data = $page->getContentHandler()->getDataForSearchIndex( $page, $output, $mockEngine );
+		$revision = $page->getRevisionRecord();
+		$output = $page->getContentHandler()->getParserOutputForIndexing( $page, null, $revision );
+		$data = $page->getContentHandler()->getDataForSearchIndex( $page, $output, $mockEngine, $revision );
 		$this->assertArrayHasKey( 'text', $data );
 		$this->assertArrayHasKey( 'text_bytes', $data );
 		$this->assertArrayHasKey( 'language', $data );
@@ -465,54 +404,60 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$this->assertEquals( 'wikitext', $data['content_model'] );
 	}
 
-	/**
-	 * @covers ContentHandler::getParserOutputForIndexing
-	 */
 	public function testParserOutputForIndexing() {
-		$title = Title::newFromText( 'Smithee', NS_MAIN );
-		$page = new WikiPage( $title );
+		$opportunisticUpdateHook =
+			$this->createMock( OpportunisticLinksUpdateHook::class );
+		// WikiPage::triggerOpportunisticLinksUpdate should not be triggered when
+		// getParserOutputForIndexing is called
+		$opportunisticUpdateHook->expects( $this->never() )
+			->method( 'onOpportunisticLinksUpdate' )
+			->willReturn( false );
+		$this->setTemporaryHook( 'OpportunisticLinksUpdate', $opportunisticUpdateHook );
 
-		$out = $page->getContentHandler()->getParserOutputForIndexing( $page );
+		$title = Title::makeTitle( NS_MAIN, 'Smithee' );
+		$page = $this->getServiceContainer()->getWikiPageFactory()->newFromTitle( $title );
+		$revision = $page->getRevisionRecord();
+
+		$out = $page->getContentHandler()->getParserOutputForIndexing( $page, null, $revision );
 		$this->assertInstanceOf( ParserOutput::class, $out );
 		$this->assertStringContainsString( 'one who smiths', $out->getRawText() );
 	}
 
-	/**
-	 * @covers ContentHandler::getContentModels
-	 */
 	public function testGetContentModelsHook() {
 		$this->setTemporaryHook( 'GetContentModels', static function ( &$models ) {
 			$models[] = 'Ferrari';
 		} );
+		$this->hideDeprecated( 'MediaWiki\\Content\\ContentHandler::getContentModels' );
 		$this->assertContains( 'Ferrari', ContentHandler::getContentModels() );
 	}
 
-	/**
-	 * @covers ContentHandler::getSlotDiffRenderer
-	 */
 	public function testGetSlotDiffRenderer_default() {
 		$this->mergeMwGlobalArrayValue( 'wgHooks', [
 			'GetSlotDiffRenderer' => [],
 		] );
 
 		// test default renderer
-		$contentHandler = new WikitextContentHandler( CONTENT_MODEL_WIKITEXT );
+		$contentHandler = new WikitextContentHandler(
+			CONTENT_MODEL_WIKITEXT,
+			$this->createMock( TitleFactory::class ),
+			$this->createMock( ParserFactory::class ),
+			$this->createMock( GlobalIdGenerator::class ),
+			$this->createMock( LanguageNameUtils::class ),
+			$this->createMock( LinkRenderer::class ),
+			$this->createMock( MagicWordFactory::class ),
+			$this->createMock( ParsoidParserFactory::class )
+		);
 		$slotDiffRenderer = $contentHandler->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertInstanceOf( TextSlotDiffRenderer::class, $slotDiffRenderer );
 	}
 
-	/**
-	 * @covers ContentHandler::getSlotDiffRenderer
-	 */
 	public function testGetSlotDiffRenderer_bc() {
 		$this->mergeMwGlobalArrayValue( 'wgHooks', [
 			'GetSlotDiffRenderer' => [],
 		] );
 
 		// test B/C renderer
-		$customDifferenceEngine = $this->getMockBuilder( DifferenceEngine::class )
-			->disableOriginalConstructor()
-			->getMock();
+		$customDifferenceEngine = $this->createMock( DifferenceEngine::class );
 		// hack to track object identity across cloning
 		$customDifferenceEngine->objectId = 12345;
 		$customContentHandler = $this->getMockBuilder( ContentHandler::class )
@@ -521,7 +466,6 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 			->getMockForAbstractClass();
 		$customContentHandler->method( 'createDifferenceEngine' )
 			->willReturn( $customDifferenceEngine );
-		/** @var ContentHandler $customContentHandler */
 		$slotDiffRenderer = $customContentHandler->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertInstanceOf( DifferenceEngineSlotDiffRenderer::class, $slotDiffRenderer );
 		$this->assertSame(
@@ -530,18 +474,13 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		);
 	}
 
-	/**
-	 * @covers ContentHandler::getSlotDiffRenderer
-	 */
 	public function testGetSlotDiffRenderer_nobc() {
 		$this->mergeMwGlobalArrayValue( 'wgHooks', [
 			'GetSlotDiffRenderer' => [],
 		] );
 
 		// test that B/C renderer does not get used when getSlotDiffRendererInternal is overridden
-		$customDifferenceEngine = $this->getMockBuilder( DifferenceEngine::class )
-			->disableOriginalConstructor()
-			->getMock();
+		$customDifferenceEngine = $this->createMock( DifferenceEngine::class );
 		$customSlotDiffRenderer = $this->getMockBuilder( SlotDiffRenderer::class )
 			->disableOriginalConstructor()
 			->getMockForAbstractClass();
@@ -553,30 +492,24 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 			->willReturn( $customDifferenceEngine );
 		$customContentHandler2->method( 'getSlotDiffRendererInternal' )
 			->willReturn( $customSlotDiffRenderer );
-		/** @var ContentHandler $customContentHandler2 */
+		$this->hideDeprecated( 'ContentHandler::getSlotDiffRendererInternal' );
 		$slotDiffRenderer = $customContentHandler2->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertSame( $customSlotDiffRenderer, $slotDiffRenderer );
 	}
 
-	/**
-	 * @covers ContentHandler::getSlotDiffRenderer
-	 */
 	public function testGetSlotDiffRenderer_hook() {
 		$this->mergeMwGlobalArrayValue( 'wgHooks', [
 			'GetSlotDiffRenderer' => [],
 		] );
 
 		// test that the hook handler takes precedence
-		$customDifferenceEngine = $this->getMockBuilder( DifferenceEngine::class )
-			->disableOriginalConstructor()
-			->getMock();
+		$customDifferenceEngine = $this->createMock( DifferenceEngine::class );
 		$customContentHandler = $this->getMockBuilder( ContentHandler::class )
 			->setConstructorArgs( [ 'foo', [] ] )
 			->onlyMethods( [ 'createDifferenceEngine' ] )
 			->getMockForAbstractClass();
 		$customContentHandler->method( 'createDifferenceEngine' )
 			->willReturn( $customDifferenceEngine );
-		/** @var ContentHandler $customContentHandler */
 
 		$customSlotDiffRenderer = $this->getMockBuilder( SlotDiffRenderer::class )
 			->disableOriginalConstructor()
@@ -589,7 +522,6 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 			->willReturn( $customDifferenceEngine );
 		$customContentHandler2->method( 'getSlotDiffRendererInternal' )
 			->willReturn( $customSlotDiffRenderer );
-		/** @var ContentHandler $customContentHandler2 */
 
 		$customSlotDiffRenderer2 = $this->getMockBuilder( SlotDiffRenderer::class )
 			->disableOriginalConstructor()
@@ -599,13 +531,16 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 				$slotDiffRenderer = $customSlotDiffRenderer2;
 			} );
 
+		$this->hideDeprecated( 'ContentHandler::getSlotDiffRendererInternal' );
 		$slotDiffRenderer = $customContentHandler->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertSame( $customSlotDiffRenderer2, $slotDiffRenderer );
+
+		$this->hideDeprecated( 'ContentHandler::getSlotDiffRendererInternal' );
 		$slotDiffRenderer = $customContentHandler2->getSlotDiffRenderer( RequestContext::getMain() );
 		$this->assertSame( $customSlotDiffRenderer2, $slotDiffRenderer );
 	}
 
-	public function providerGetPageViewLanguage() {
+	public static function providerGetPageViewLanguage() {
 		yield [ NS_FILE, 'sr', 'sr-ec', 'sr-ec' ];
 		yield [ NS_FILE, 'sr', 'sr', 'sr' ];
 		yield [ NS_MEDIAWIKI, 'sr-ec', 'sr', 'sr-ec' ];
@@ -613,19 +548,18 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
+	 * Superseded by OutputPageTest::testGetJsVarsAboutPageLang
+	 *
 	 * @dataProvider providerGetPageViewLanguage
-	 * @covers ContentHandler::getPageViewLanguage
 	 */
 	public function testGetPageViewLanguage( $namespace, $lang, $variant, $expected ) {
 		$contentHandler = $this->getMockBuilder( ContentHandler::class )
 			->disableOriginalConstructor()
 			->getMockForAbstractClass();
 
-		$title = Title::newFromText( "SimpleTitle", $namespace );
+		$title = Title::makeTitle( $namespace, 'SimpleTitle' );
 
-		$this->setMwGlobals( [
-			'wgDefaultLanguageVariant' => $variant,
-		] );
+		$this->overrideConfigValue( MainConfigNames::DefaultLanguageVariant, $variant );
 
 		$this->setUserLang( $lang );
 		$this->setContentLang( $lang );
@@ -634,7 +568,7 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$this->assertEquals( $expected, $pageViewLanguage->getCode() );
 	}
 
-	public function provideValidateSave() {
+	public static function provideValidateSave() {
 		yield 'wikitext' => [
 			new WikitextContent( 'hello world' ),
 			true
@@ -653,7 +587,6 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 
 	/**
 	 * @dataProvider provideValidateSave
-	 * @covers ContentHandler::validateSave
 	 */
 	public function testValidateSave( $content, $expectedResult ) {
 		$page = new PageIdentityValue( 0, 1, 'Foo', PageIdentity::LOCAL );
@@ -662,6 +595,6 @@ class ContentHandlerTest extends MediaWikiIntegrationTestCase {
 		$validateParams = new ValidationParams( $page, 0 );
 
 		$status = $contentHandler->validateSave( $content, $validateParams );
-		$this->assertEquals( $expectedResult, $status->isOk() );
+		$this->assertEquals( $expectedResult, $status->isOK() );
 	}
 }
