@@ -1,20 +1,36 @@
 <?php
 
-use MediaWiki\Block\BlockManager;
-use MediaWiki\Block\DatabaseBlock;
+use MediaWiki\Block\UserBlockTarget;
+use MediaWiki\Context\RequestContext;
+use MediaWiki\HookContainer\HookContainer;
+use MediaWiki\Interwiki\InterwikiLookup;
 use MediaWiki\Linker\LinkTarget;
+use MediaWiki\MainConfigNames;
+use MediaWiki\Output\OutputPage;
 use MediaWiki\Page\PageReferenceValue;
-use MediaWiki\Permissions\Authority;
+use MediaWiki\Request\FauxRequest;
+use MediaWiki\Skin\Skin;
+use MediaWiki\Skin\SkinFallback;
+use MediaWiki\Tests\Unit\MockBlockTrait;
 use MediaWiki\Tests\Unit\Permissions\MockAuthorityTrait;
+use MediaWiki\Tests\User\TempUser\TempUserTestTrait;
 use MediaWiki\Title\Title;
+use MediaWiki\Title\TitleValue;
 use MediaWiki\User\UserIdentity;
+use MediaWiki\User\UserIdentityLookup;
 use MediaWiki\User\UserIdentityValue;
 
+/**
+ * @covers \MediaWiki\Skin\Skin
+ * @group Database
+ */
 class SkinTest extends MediaWikiIntegrationTestCase {
 	use MockAuthorityTrait;
+	use MockBlockTrait;
+	use TempUserTestTrait;
 
 	/**
-	 * @covers Skin
+	 * @covers \MediaWiki\Skin\Skin
 	 */
 	public function testGetSkinName() {
 		$skin = new SkinFallback();
@@ -23,9 +39,6 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		$this->assertEquals( 'testname', $skin->getSkinName(), 'Constructor argument' );
 	}
 
-	/**
-	 * @covers Skin::getDefaultModules
-	 */
 	public function testGetDefaultModules() {
 		$skin = $this->getMockBuilder( Skin::class )
 			->onlyMethods( [ 'outputPage' ] )
@@ -36,22 +49,85 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		$this->assertTrue( isset( $modules['styles'] ), 'style key is set by default' );
 	}
 
-	public function provideGetDefaultModulesWatchWrite() {
+	/**
+	 * @param bool $isSyndicated
+	 * @param string $html
+	 * @return OutputPage
+	 */
+	private function getMockOutputPage( $isSyndicated, $html ) {
+		$mock = $this->createMock( OutputPage::class );
+		$mock->expects( $this->once() )
+			->method( 'isSyndicated' )
+			->willReturn( $isSyndicated );
+		$mock->method( 'getHTML' )
+			->willReturn( $html );
+		return $mock;
+	}
+
+	public static function provideGetDefaultModulesForOutput() {
+		return [
+			[
+				false,
+				'',
+				[]
+			],
+			[
+				true,
+				'',
+				[ 'mediawiki.feedlink' ]
+			],
+			[
+				false,
+				'FOO mw-ui-button BAR',
+				[ 'mediawiki.ui.button' ]
+			],
+			[
+				true,
+				'FOO mw-ui-button BAR',
+				[ 'mediawiki.ui.button', 'mediawiki.feedlink' ]
+			],
+		];
+	}
+
+	/**
+	 * @dataProvider provideGetDefaultModulesForOutput
+	 */
+	public function testGetDefaultModulesForContent( $isSyndicated, $html, array $expectedModuleStyles ) {
+		$skin = new class extends Skin {
+			public function outputPage() {
+			}
+		};
+		$fakeContext = new RequestContext();
+		$fakeContext->setTitle( Title::makeTitle( NS_MAIN, 'Test' ) );
+		$fakeContext->setOutput( $this->getMockOutputPage( $isSyndicated, $html ) );
+		$skin->setContext( $fakeContext );
+
+		$modules = $skin->getDefaultModules();
+
+		$actualStylesModule = array_merge( ...array_values( $modules['styles'] ) );
+		foreach ( $expectedModuleStyles as $expected ) {
+			$this->assertContains( $expected, $actualStylesModule );
+		}
+	}
+
+	public static function provideGetDefaultModulesForRights() {
 		yield 'no rights' => [
-			$this->mockRegisteredNullAuthority(), // $authority
+			'null', // $authoritySpec
 			false, // $hasModule
 		];
 		yield 'has all rights' => [
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			true, // $hasModule
 		];
 	}
 
 	/**
-	 * @dataProvider provideGetDefaultModulesWatchWrite
-	 * @covers Skin::getDefaultModules
+	 * @dataProvider provideGetDefaultModulesForRights
 	 */
-	public function testGetDefaultModulesWatchWrite( Authority $authority, bool $hasModule ) {
+	public function testGetDefaultModulesForRights( string $authoritySpec, bool $hasModule ) {
+		$authority = $authoritySpec === 'ultimate'
+			? $this->mockRegisteredUltimateAuthority()
+			: $this->mockRegisteredNullAuthority();
 		$skin = new class extends Skin {
 			public function outputPage() {
 			}
@@ -70,55 +146,61 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		}
 	}
 
-	public function providGetPageClasses() {
+	public static function provideGetPageClasses() {
 		yield 'normal page has namespace' => [
 			new TitleValue( NS_MAIN, 'Test' ), // $title
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			[ 'ns-0' ], // $expectedClasses
 		];
 		yield 'valid special page' => [
 			new TitleValue( NS_SPECIAL, 'Userlogin' ), // $title
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			[ 'mw-special-Userlogin' ], // $expectedClasses
 		];
 		yield 'invalid special page' => [
 			new TitleValue( NS_SPECIAL, 'BLABLABLABLA_I_AM_INVALID' ), // $title
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			[ 'mw-invalidspecialpage' ], // $expectedClasses
 		];
 		yield 'talk page' => [
 			new TitleValue( NS_TALK, 'Test' ), // $title
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			[ 'ns-talk' ], // $expectedClasses
 		];
 		yield 'subject' => [
 			new TitleValue( NS_MAIN, 'Test' ), // $title
-			$this->mockRegisteredUltimateAuthority(), // $authority
+			'ultimate', // $authoritySpec
 			[ 'ns-subject' ], // $expectedClasses
 		];
 		yield 'editable' => [
 			new TitleValue( NS_MAIN, 'Test' ), // $title
-			$this->mockRegisteredAuthorityWithPermissions( [ 'edit' ] ), // $authority
+			[ 'edit' ], // $authoritySpec
 			[ 'mw-editable' ], // $expectedClasses
 		];
 		yield 'not editable' => [
 			new TitleValue( NS_MAIN, 'Test' ), // $title
-			$this->mockRegisteredNullAuthority(), // $authority
+			'null', // $authoritySpec
 			[], // $expectedClasses
 			[ 'mw-editable' ], // $unexpectedClasses
 		];
 	}
 
 	/**
-	 * @dataProvider providGetPageClasses
-	 * @covers Skin::getPageClasses
+	 * @dataProvider provideGetPageClasses
 	 */
 	public function testGetPageClasses(
 		LinkTarget $title,
-		Authority $authority,
+		$authoritySpec,
 		array $expectedClasses,
 		array $unexpectedClasses = []
 	) {
+		if ( is_array( $authoritySpec ) ) {
+			$authority = $this->mockRegisteredAuthorityWithPermissions( $authoritySpec );
+		} else {
+			$authority = $authoritySpec === 'ultimate'
+				? $this->mockRegisteredUltimateAuthority()
+				: $this->mockRegisteredNullAuthority();
+		}
 		$skin = new class extends Skin {
 			public function outputPage() {
 			}
@@ -136,8 +218,6 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers Skin::isResponsive
-	 *
 	 * @dataProvider provideSkinResponsiveOptions
 	 */
 	public function testIsResponsive( array $options, bool $expected ) {
@@ -166,7 +246,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		$this->assertSame( $expected, $skin->isResponsive() );
 	}
 
-	public function provideSkinResponsiveOptions() {
+	public static function provideSkinResponsiveOptions() {
 		yield 'responsive not set' => [
 			[ 'name' => 'test', 'userPreference' => true ],
 			false
@@ -189,10 +269,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		];
 	}
 
-	/**
-	 * @covers Skin::makeLink
-	 */
-	public function provideMakeLink() {
+	public static function provideMakeLink() {
 		return [
 			'Empty href with link class' => [
 				[
@@ -293,7 +370,6 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers Skin::makeLink
 	 * @dataProvider provideMakeLink
 	 * @param array $data
 	 * @param array $options
@@ -318,7 +394,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		);
 	}
 
-	public function provideGetPersonalToolsForMakeListItem() {
+	public static function provideGetPersonalToolsForMakeListItem() {
 		return [
 			[
 				[
@@ -340,6 +416,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 							]
 						],
 						'id' => 'pt-foo',
+						'icon' => null,
 					]
 				],
 			],
@@ -362,6 +439,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 							]
 						],
 						'id' => 'pt-foo',
+						'icon' => null,
 						'class' => 'foo',
 					]
 				],
@@ -370,11 +448,10 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers Skin::getPersonalToolsForMakeListItem
 	 * @dataProvider provideGetPersonalToolsForMakeListItem
 	 * @param array $urls
-	 * @param array $applyClassesToListItems
-	 * @param string $expected
+	 * @param bool $applyClassesToListItems
+	 * @param array $expected
 	 */
 	public function testGetPersonalToolsForMakeListItem( array $urls, bool $applyClassesToListItems, array $expected ) {
 		$skin = new class extends Skin {
@@ -391,29 +468,22 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		);
 	}
 
-	/**
-	 * @covers Skin::setRelevantUser
-	 * @covers Skin::getRelevantUser
-	 */
 	public function testGetRelevantUser_get_set() {
 		$skin = new class extends Skin {
 			public function outputPage() {
 			}
 		};
-		$relevantUser = UserIdentityValue::newRegistered( 1, '123.123.123.123' );
+		$relevantUser = UserIdentityValue::newRegistered( 1, 'SomeUser' );
 		$skin->setRelevantUser( $relevantUser );
 		$this->assertSame( $relevantUser, $skin->getRelevantUser() );
 
-		$blockManagerMock = $this->createNoOpMock( BlockManager::class, [ 'getUserBlock' ] );
-		$blockManagerMock->method( 'getUserBlock' )
-			->with( $relevantUser )
-			->willReturn( new DatabaseBlock( [
-				'address' => $relevantUser,
-				'wiki' => $relevantUser->getWikiId(),
-				'by' => UserIdentityValue::newAnonymous( '123.123.123.123' ),
-				'hideName' => true
-			] ) );
-		$this->setService( 'BlockManager', $blockManagerMock );
+		$this->installMockBlockManager(
+			[
+				'target' => new UserBlockTarget( $relevantUser ),
+				'hideName' => true,
+			]
+		);
+
 		$ctx = RequestContext::getMain();
 		$ctx->setAuthority( $this->mockAnonNullAuthority() );
 		$skin->setContext( $ctx );
@@ -426,7 +496,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		$this->assertSame( $relevantUser, $skin->getRelevantUser() );
 	}
 
-	public function provideGetRelevantUser_load_from_title() {
+	public static function provideGetRelevantUser_load_from_title() {
 		yield 'Not user namespace' => [
 			'relevantPage' => PageReferenceValue::localReference( NS_MAIN, '123.123.123.123' ),
 			'expectedUser' => null
@@ -451,7 +521,6 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 
 	/**
 	 * @dataProvider provideGetRelevantUser_load_from_title
-	 * @covers Skin::getRelevantUser
 	 */
 	public function testGetRelevantUser_load_from_title(
 		PageReferenceValue $relevantPage,
@@ -461,7 +530,7 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 			public function outputPage() {
 			}
 		};
-		$skin->setRelevantTitle( Title::castFromPageReference( $relevantPage ) );
+		$skin->setRelevantTitle( Title::newFromPageReference( $relevantPage ) );
 		$relevantUser = $skin->getRelevantUser();
 		if ( $expectedUser ) {
 			$this->assertTrue( $expectedUser->equals( $relevantUser ) );
@@ -470,19 +539,220 @@ class SkinTest extends MediaWikiIntegrationTestCase {
 		}
 	}
 
-	/**
-	 * @covers Skin::getRelevantUser
-	 */
 	public function testGetRelevantUser_load_existing() {
 		$skin = new class extends Skin {
 			public function outputPage() {
 			}
 		};
-		$existingUser = $this->getTestSysop()->getUserIdentity();
+		$user = new UserIdentityValue( 42, 'foo' );
+		$userIdentityLookup = $this->createMock( UserIdentityLookup::class );
+		$userIdentityLookup->method( 'getUserIdentityByName' )
+			->willReturnCallback( function ( $name ) use ( $user ) {
+				if ( $name === $user->getName() ) {
+					return $user;
+				}
+				return $this->createMock( UserIdentity::class );
+			} );
+		$this->setService( 'UserIdentityLookup', $userIdentityLookup );
 		$skin->setRelevantTitle(
-			Title::makeTitle( NS_USER, $this->getTestSysop()->getUserIdentity()->getName() )
+			Title::makeTitle( NS_USER, $user->getName() )
 		);
-		$this->assertTrue( $existingUser->equals( $skin->getRelevantUser() ) );
-		$this->assertSame( $existingUser->getId(), $skin->getRelevantUser()->getId() );
+		$this->assertTrue( $user->equals( $skin->getRelevantUser() ) );
+		$this->assertSame( $user->getId(), $skin->getRelevantUser()->getId() );
+	}
+
+	public function testBuildSidebarCache() {
+		// T303007: Skin subclasses and Skin hooks may vary their sidebar contents.
+		$this->overrideConfigValues( [
+			MainConfigNames::UseDatabaseMessages => true,
+			MainConfigNames::EnableSidebarCache => true,
+			MainConfigNames::SidebarCacheExpiry => 3600,
+		] );
+		// Mock time (T344191)
+		$clock = 1301644800.0;
+		$this->getServiceContainer()->getMainWANObjectCache()->setMockTime( $clock );
+		$id = 0;
+		$this->setTemporaryHook( 'SkinBuildSidebar',
+			static function ( Skin $skin, array &$bar ) use ( &$id, &$clock ) {
+				$id++;
+				$clock += 1.0;
+				if ( $skin->getSkinName() === 'foo' ) {
+					$bar['myhook'] = "foo $id";
+				}
+				if ( $skin->getSkinName() === 'bar' ) {
+					$bar['myhook'] = "bar $id";
+				}
+			}
+		);
+		$context = RequestContext::newExtraneousContext( Title::makeTitle( NS_SPECIAL, 'Blankpage' ) );
+		$foo1 = new class( 'foo' ) extends Skin {
+			public function outputPage() {
+			}
+		};
+		$foo2 = new class( 'foo' ) extends Skin {
+			public function outputPage() {
+			}
+		};
+		$bar = new class( 'bar' ) extends Skin {
+			public function outputPage() {
+			}
+		};
+		$foo1->setContext( $context );
+		$foo2->setContext( $context );
+		$bar->setContext( $context );
+		$this->assertArrayContains( [ 'myhook' => 'foo 1' ], $foo1->buildSidebar(), 'fresh' );
+		$clock += 0.01;
+		$this->assertArrayContains( [ 'myhook' => 'foo 1' ], $foo2->buildSidebar(), 'cache hit' );
+		$this->assertArrayContains( [ 'myhook' => 'bar 2' ], $bar->buildSidebar(), 'cache miss' );
+	}
+
+	public function testBuildSidebarWithUserAddedContent() {
+		$this->overrideConfigValues( [
+			MainConfigNames::UseDatabaseMessages => true,
+			MainConfigNames::EnableSidebarCache => false
+		] );
+		$foo1 = new class( 'foo' ) extends Skin {
+			public function outputPage() {
+			}
+		};
+		$this->editPage( 'MediaWiki:Sidebar', <<<EOS
+		* navigation
+		** mainpage|mainpage-description
+		** recentchanges-url|recentchanges
+		** randompage-url|randompage
+		** helppage|help-mediawiki
+		** specialpages-url|specialpages
+		* SEARCH
+		* TOOLBOX
+		** A|B
+		* LANGUAGES
+		** C|D
+		EOS );
+
+		$context = RequestContext::newExtraneousContext( Title::makeTitle( NS_MAIN, 'Main Page' ) );
+		$foo1->setContext( $context );
+
+		$this->assertArrayContains( [ [ 'id' => 'n-B', 'text' => 'B' ] ], $foo1->buildSidebar()['TOOLBOX'], 'Toolbox has user defined links' );
+
+		$hasUserDefinedLinks = false;
+		$languageLinks = $foo1->buildSidebar()['LANGUAGES'];
+		foreach ( $languageLinks as $languageLink ) {
+			if ( $languageLink['id'] === 'n-D' ) {
+				$hasUserDefinedLinks = true;
+				break;
+			}
+		}
+
+		$this->assertSame( false, $hasUserDefinedLinks, 'Languages does not support user defined links' );
+	}
+
+	public function testBuildSidebarForContributionsPageOfTemporaryAccount() {
+		// Don't allow extensions to modify the TOOLBOX array as we assert pretty strictly against it.
+		$this->clearHook( 'SidebarBeforeOutput' );
+
+		$this->overrideConfigValues( [
+			MainConfigNames::UploadNavigationUrl => false,
+			MainConfigNames::EnableUploads => false,
+			MainConfigNames::EnableSpecialMute => true,
+		] );
+		$foo1 = new class( 'foo' ) extends Skin {
+			public function outputPage() {
+			}
+		};
+
+		// Simulate the settings and context for Special:Contributions for a temporary account
+		// (no article related and relevant user set).
+		$this->enableAutoCreateTempUser();
+		$tempUser = $this->getServiceContainer()->getTempUserCreator()
+			->create( null, new FauxRequest() )->getUser();
+		$context = RequestContext::newExtraneousContext(
+			Title::makeTitle( NS_SPECIAL, 'Contributions/' . $tempUser->getName() )
+		);
+		$context->setUser( $this->getTestSysop()->getUser() );
+		$foo1->setContext( $context );
+		$foo1->setRelevantUser( $tempUser );
+		$foo1->getOutput()->setArticleRelated( false );
+
+		// Verify that the "userrights" key is not present, by checking that the list of keys is as expected.
+		$this->assertArrayEquals(
+			[ 'contributions', 'log', 'blockip', 'mute', 'print' ],
+			array_keys( $foo1->buildSidebar()['TOOLBOX'] )
+		);
+	}
+
+	public function testGetLanguagesHidden() {
+		$this->overrideConfigValues( [
+			MainConfigNames::HideInterlanguageLinks => true,
+		] );
+		$skin = new class extends Skin {
+			public function outputPage() {
+			}
+		};
+		$this->assertSame( [], $skin->getLanguages() );
+	}
+
+	public function testGetLanguages() {
+		$this->overrideConfigValues( [
+			MainConfigNames::HideInterlanguageLinks => false,
+			MainConfigNames::InterlanguageLinkCodeMap => [ 'talk' => 'fr' ],
+			MainConfigNames::LanguageCode => 'qqx',
+		] );
+
+		$mockOutputPage = $this->createMock( OutputPage::class );
+		$mockOutputPage->method( 'getLanguageLinks' )
+			// The 'talk' interwiki is a deliberate conflict with the
+			// Talk namespace (T363538)
+			->willReturn( [ 'en:Foo', 'talk:Page' ] );
+
+		$fakeContext = new RequestContext();
+		$fakeContext->setTitle( Title::makeTitle( NS_MAIN, 'Test' ) );
+		$fakeContext->setOutput( $mockOutputPage );
+		$fakeContext->setLanguage( 'en' );
+
+		$hookContainer = $this->createMock( HookContainer::class );
+		$this->setService( 'HookContainer', $hookContainer );
+
+		$mockIwLookup = $this->createMock( InterwikiLookup::class );
+		$mockIwLookup->method( 'isValidInterwiki' )->willReturn( true );
+		$mockIwLookup->method( 'fetch' )->willReturnCallback( static function ( string $prefix ) {
+			return new Interwiki(
+				$prefix,
+				"https://$prefix.example.com/$1"
+			);
+		} );
+		$this->setService( 'InterwikiLookup', $mockIwLookup );
+
+		$skin = new class extends Skin {
+			public function outputPage() {
+			}
+		};
+		$skin->setContext( $fakeContext );
+
+		$this->assertSame( [
+			[
+				'href' => 'https://en.example.com/Foo',
+				'text' => 'English',
+				'title' => 'Foo – English',
+				'class' => 'interlanguage-link interwiki-en',
+				'link-class' => 'interlanguage-link-target',
+				'lang' => 'en',
+				'hreflang' => 'en',
+				'data-title' => 'Foo',
+				'data-language-autonym' => 'English',
+				'data-language-local-name' => 'English',
+			],
+			[
+				'href' => 'https://talk.example.com/Page',
+				'text' => 'Français',
+				'title' => 'Page – français',
+				'class' => 'interlanguage-link interwiki-talk',
+				'link-class' => 'interlanguage-link-target',
+				'lang' => 'fr',
+				'hreflang' => 'fr',
+				'data-title' => 'Page',
+				'data-language-autonym' => 'Français',
+				'data-language-local-name' => 'français',
+			],
+		], $skin->getLanguages() );
 	}
 }
